import pygame, sys
from pygame.locals import *

levelPath = "res/levels/"
tilePath = "res/tiles/"
objectPath = "res/objects/"
spritePath = "res/sprites/"

tilewidth = 16

pygame.init()

class Window:
	def __init__(self,screenRes,GUIres):
		self.window = pygame.display.set_mode(GUIres)
		self.screen = pygame.Surface(screenRes)
		self.gui = pygame.Surface(GUIres)
		self.gui.set_colorkey((255,255,255))
		self.x_offset = 0
		self.y_offset = 0
		self.screen.fill((0,0,0))
		self.gui.fill((255,255,255))
		
	def update(self):
		self.window.blit(self.screen, (0,0))
		self.window.blit(self.gui, (0,0))
		pygame.display.flip()
		self.window.fill((0,0,0))
		self.screen.fill((0,0,0))
		self.gui.fill((255,255,255))
		
	def blitScreen(self,image,pos):
		self.screen.blit(image,(pos[0]+self.x_offset,pos[1]+self.y_offset))
		
	def blitGUI(self,image,pos):
		self.gui.blit(image,pos)	
		
class Tile:
	def __init__(self,name,transparent):
		self.name = name
		self.transparent = transparent
		self.image = pygame.image.load(tilePath+self.name+".png")
		self.pos = pygame.Rect(0,0,tilewidth,tilewidth)
		
class Object:
	def __init__(self,name,pos):
		self.name = name
		self.image = pygame.image.load(objectPath+self.name+".png")
		self.image.set_colorkey((0,0,0))
		self.pos = pos
		
class Antagonist:
	def __init__(self,pos,AI):
		self.image = pygame.image.load(spritePath+"clown_stand.png")
		self.image.set_colorkey((0,0,0))
		self.pos = pos
		self.AI = AI

if len(sys.argv) > 2:
	print "Too many files"
	sys.exit()
	
if len(sys.argv) == 1:
	print "No file specified"
	sys.exit()
		
fileName = levelPath + sys.argv[1]

#set up tiles

tiles = []

f = open(tilePath+"description.des", 'r')
lines = f.readlines()
f.close()

for line in lines:
	line = line.split(" ")
	tiles.append(Tile(line[0],(line[1] == "transparent" or line[1] == "transparent\n")))
	
objects = []

f = open(objectPath+"description.des", 'r')
lines = f.readlines()
f.close()

for line in lines:
	line = line.split(" ")
	objects.append(Object(line[0],pygame.Rect(0,0,int(line[1]),int(line[2]))))

#set up save	
	
level = []
levelObjects = []
levelAntagonists = []
protagonistPos = (0,0)
piePos = (0,0)

saved = True

def save():
	print "Saving..."

	f = open(fileName, 'w')
	
	f.write(str(len(level))+"\n")
	
	for row in level:
		line = ""
		for tile in row:
			line += tile.name + " "
		f.write(line+"\n")
	
	f.write(str(len(levelObjects))+"\n")
	for object in levelObjects:
		f.write(object.name+" "+str(object.pos[0])+" "+str(object.pos[1])+"\n")
		
	f.write(str(len(levelAntagonists))+"\n")
	for a in levelAntagonists:
		line = str(a.pos[0])+" "+str(a.pos[1])
		for step in a.AI:
			line += " " + str(step[0])
		f.write(line+"\n")
	
	f.write(str(protagonistPos[0])+" "+str(protagonistPos[1])+" "+"\n")	
	f.write(str(piePos[0])+" "+str(piePos[1])+" "+"\n")
				
	f.close()
	
	print "Saved"
	saved = True

#test if level exists	
	
try:
	f = open(fileName, 'r')
	print "Opening " + fileName
except:
	print fileName + " does not exist. Creating..."
	
	for i in range(16):
		row = [];
		for j in range(16):
			row.append(tiles[0])
		level.append(row)
		
	save()
	f = open(fileName, 'r')

#open level	
	
lines = f.readlines()
f.close()

for i in range(int(lines[0])):
	line = lines[1+i].split(" ")
	row = []
	for tile in line:
		try:
			row.append(tiles[int(tile)])
		except:
			pass
	level.append(row)

start = int(lines[0])+1		
for i in range(start,start+int(lines[start])):
	line = lines[1+i].split(" ")
	levelObjects.append(Object(line[0],pygame.Rect(int(line[1]),int(line[2]),0,0)))
	for object in objects:
		if object.name == levelObjects[-1].name:
			levelObjects[-1].pos[2] = object.pos[2]
			levelObjects[-1].pos[3] = object.pos[3]
			break
			
start += int(lines[start])+1
	
for i in range(start,start+int(lines[start])):
	line = lines[1+i].split(" ")
	ai = line[2:]
	for i in range(len(ai)):
		ai[i] = (int(ai[i]),0)
	levelAntagonists.append(Antagonist(pygame.Rect(int(line[0]),int(line[1]),tilewidth,tilewidth),ai))
	
index = start+int(lines[start])+1

line = lines[index].split(" ")

protagonistPos = (int(line[0]),int(line[1]))

index += 1

line = lines[index].split(" ")

piePos = (int(line[0]),int(line[1]))

#set up GUI	
	
window = Window((500,500),(500,600))	

arrowRight = pygame.image.load(levelPath+"arrow.png")
arrowLeft = pygame.transform.flip(arrowRight,1,0)

arrowLrec = pygame.Rect(tilewidth,500+5*tilewidth,tilewidth,tilewidth)
arrowRrec = pygame.Rect(500-2*tilewidth,500+5*tilewidth,tilewidth,tilewidth)

protagonistIcon = pygame.image.load(spritePath+"dude_stand.png")
protagonistIcon.set_colorkey((255,255,255))
protagonistRect = pygame.Rect(tilewidth*3,500+5*tilewidth,tilewidth,tilewidth)

antagonistIcon = pygame.image.load(spritePath+"clown_stand.png")
antagonistIcon.set_colorkey((255,255,255))
antagonistRect = pygame.Rect(tilewidth*5,500+5*tilewidth,tilewidth,tilewidth)

pieIcon = pygame.image.load(objectPath+"pie.png")
pieIcon.set_colorkey((255,255,255))
pieRect = pygame.Rect(tilewidth*7,500+5*tilewidth,tilewidth,tilewidth)

node = pygame.image.load(levelPath+"node.png")
node.set_colorkey((255,255,255))

jumpNode = pygame.image.load(levelPath+"jump.png")
jumpNode.set_colorkey((255,255,255))

tileSelectionOffset = 0

ctrlPressed = False

selectedTile = -1
selectedObject = -1
antagonistSelect = False
antagonistAI = False
protagonistMove = False
pieMove = False
dragging = False
dragRect = pygame.Rect(0,0,0,0)

selectRects = []
objectSelectRects = []

def setUpTiles():
	for i in range(len(tiles)):
		tiles[i].pos[0]=tileSelectionOffset*tilewidth*2+i*tilewidth*2+tilewidth
		tiles[i].pos[1]=500+tilewidth

	for i in range(len(objects)):
		objects[i].pos[0]=tileSelectionOffset*tilewidth*2+i*tilewidth*2+tilewidth
		objects[i].pos[1]=500+tilewidth*3

setUpTiles()		
				
while True:
	
	#draw screen
	
	for i in range(len(tiles)):
		window.blitGUI(tiles[i].image,tiles[i].pos)
		
	for object in objects:
		window.blitGUI(object.image,object.pos)
		
	window.blitGUI(arrowLeft,arrowLrec)
	window.blitGUI(arrowRight,arrowRrec)
	
	window.blitGUI(protagonistIcon,protagonistRect)
	window.blitGUI(antagonistIcon,antagonistRect)
	window.blitGUI(pieIcon,pieRect)
	
	for y in range(len(level)):
		for x in range(len(level[y])):
			window.blitScreen(level[y][x].image,(x*tilewidth,y*tilewidth))
			
	for object in levelObjects:
		window.blitScreen(object.image,object.pos)
	
	for antagonist in levelAntagonists:
		window.blitScreen(antagonist.image,antagonist.pos)
	
	if protagonistMove:
		window.blitScreen(protagonistIcon,(pygame.mouse.get_pos()[0]-window.x_offset,pygame.mouse.get_pos()[1]-window.y_offset))
	else:
		window.blitScreen(protagonistIcon,protagonistPos)
		
	if pieMove:
		window.blitScreen(pieIcon,(pygame.mouse.get_pos()[0]-window.x_offset,pygame.mouse.get_pos()[1]-window.y_offset))
	else:
		window.blitScreen(pieIcon,piePos)
	
	if dragging:
		if dragRect[2] > 0:
			xStart = dragRect[0]
			xLen = dragRect[2]
		else:
			xStart = dragRect[0]+dragRect[2]
			xLen = 0-dragRect[2]
			
		if dragRect[3] > 0:
			yStart = dragRect[1]
			yLen = dragRect[3]
		else:
			yStart = dragRect[1]+dragRect[3]
			yLen = 0-dragRect[3]
	
		#window.blitScreen(tiles[selectedTile].image,(xStart-xStart%tilewidth-window.x_offset,yStart-yStart%tilewidth-window.y_offset))	
		if xLen == 0:
			xLen = 1
			
		if yLen == 0:
			yLen = 1
		
		for i in range(xStart,xStart+xLen,tilewidth):
			for j in range(yStart,yStart+yLen,tilewidth):	
				window.blitScreen(tiles[selectedTile].image,(i-i%tilewidth-window.x_offset,j-j%tilewidth-window.y_offset))
			
		"""
		for i in range(dragRect[0],dragRect[0]+dragRect[2],tilewidth):
			for j in range(dragRect[1],dragRect[1]+dragRect[3],tilewidth):	
				window.blitScreen(tiles[selectedTile].image,(i-i%tilewidth-window.x_offset,j-j%tilewidth-window.y_offset))
		"""
				
	elif selectedTile != -1:
		pos = pygame.mouse.get_pos()
		window.blitScreen(tiles[selectedTile].image,(pos[0]-pos[0]%tilewidth-window.x_offset,pos[1]-pos[1]%tilewidth-window.y_offset))
		
	elif selectedObject != -1:
		pos = pygame.mouse.get_pos()
		window.blitScreen(objects[selectedObject].image,(pos[0]-window.x_offset,pos[1]-window.y_offset))
		
	elif antagonistSelect:
		if antagonistAI:
			for n in levelAntagonists[-1].AI:
				if n[0] == -1:
					window.blitScreen(jumpNode,n)
				else:
					window.blitScreen(node,n)
		else:
			pos = pygame.mouse.get_pos()
			window.blitScreen(antagonistIcon,(pos[0]-window.x_offset,pos[1]-window.y_offset))
					
	window.update()
	
	#handle events
	
	for event in pygame.event.get():
		if event.type == KEYUP:
			if event.key == K_ESCAPE:
				selectedTile = -1
				selectedObject = -1
				antagonistSelect = False
				dragging = False
				antagonistAI = False
				protagonistMove = False
				pieMove = False
				
			if event.key == K_LCTRL or event.key == K_RCTRL:
				ctrlPressed = False
			
		if event.type == KEYDOWN:		
					
			if event.key == K_UP:
				window.y_offset += tilewidth
				
			if event.key == K_DOWN:
				window.y_offset -= tilewidth
				
			if event.key == K_LEFT:
				window.x_offset += tilewidth
				
			if event.key == K_RIGHT:
				window.x_offset -= tilewidth
				
			if event.key == ord('s'):
				save()
			
			if event.key == K_LCTRL or event.key == K_RCTRL:
				ctrlPressed = True	
					
			if event.key == K_KP8 or event.key == K_u:
				if ctrlPressed:
					if len(level) > 1:
						level = level[1:]
				else:
					newRow = []
					for i in range(len(level[0])):
						newRow.append(tiles[0])
					level = [newRow] + level
					
				saved = False
				
			if event.key == K_KP4 or event.key == K_h:
				if ctrlPressed:
					if len(level[0]) > 1:
						for i in range(len(level)):
							level[i] = level[i][1:]
				else:
					for i in range(len(level)):
						level[i] = [tiles[0]] + level[i]
					
				saved = False
				
			if event.key == K_KP2 or event.key == K_j:
				if ctrlPressed:
					if len(level) > 1:
						level = level[:-1]
				else:
					newRow = []
					for i in range(len(level[0])):
						newRow.append(tiles[0])
					level.append(newRow)
					
				saved = False
				
			if event.key == K_KP6 or event.key == K_k:
				if ctrlPressed:
					if len(level[0]) > 1:
						for i in range(len(level)):
							level[i] = level[i][:-1]
				else:
					for i in range(len(level)):
						level[i].append(tiles[0])
					
				saved = False
		
		if event.type == MOUSEMOTION:
			if dragging:
				dragRect[2] = event.pos[0]-dragRect[0]
				dragRect[3] = event.pos[1]-dragRect[1]
				"""if dragRect[2] < 0:
					dragRect[0] = event.pos[0]
					dragRect[2] *= -1	
				
				if dragRect[3] < 0:
					dragRect[1] = event.pos[0]
					dragRect[3] *= -1
				"""			
				
		if event.type == MOUSEBUTTONDOWN:
			if dragging:
				dragRect[0] = event.pos[0]
				dragRect[1] = event.pos[1]	
			
			elif selectedTile != -1:
				dragRect[0] = event.pos[0];
				dragRect[1] = event.pos[1]
				dragRect[2] = event.pos[0]-dragRect[0]
				dragRect[3] = event.pos[1]-dragRect[1]
				dragging = True	
					
			elif event.pos[1] > 500:
				dragging = False
				antagonistSelect = False
				selectedObject = -1
				selectedTile = -1	
				antagonistAI = False
				protagonistMove = False		
				pieMove = False
			
				if arrowLrec.collidepoint(event.pos):
					tileSelectionOffset += 1
					setUpTiles()
					
				elif arrowRrec.collidepoint(event.pos):
					tileSelectionOffset -= 1
					setUpTiles()
				
				elif antagonistRect.collidepoint(event.pos):
					antagonistSelect = True	
					
				elif protagonistRect.collidepoint(event.pos):
					protagonistMove = True	
					
				elif pieRect.collidepoint(event.pos):
					pieMove = True	
					
				else:
					for i in range(len(tiles)):
						if tiles[i].pos.collidepoint(event.pos):
							selectedTile = i 
							selectedObject = -1
							
					for i in range(len(objects)):
						if objects[i].pos.collidepoint(event.pos):
							selectedObject = i
							selectedTile = -1
					
		if event.type == MOUSEBUTTONUP:
			if dragging:
						
				if dragRect[2] > 0:
					xStart = dragRect[0]
					xLen = dragRect[2]
				else:
					xStart = dragRect[0]+dragRect[2]
					xLen = 0-dragRect[2]
					
				if dragRect[3] > 0:
					yStart = dragRect[1]
					yLen = dragRect[3]
				else:
					yStart = dragRect[1]+dragRect[3]
					yLen = 0-dragRect[3]
			
				#window.blitScreen(tiles[selectedTile].image,(xStart-xStart%tilewidth-window.x_offset,yStart-yStart%tilewidth-window.y_offset))	
				if xLen == 0:
					xLen = 1
					
				if yLen == 0:
					yLen = 1
				
				for i in range(xStart,xStart+xLen,tilewidth):
					for j in range(yStart,yStart+yLen,tilewidth):
						try:	
							level[(j-j%tilewidth-window.y_offset)/tilewidth][(i-i%tilewidth-window.x_offset)/tilewidth] = tiles[selectedTile]
						except:
							pass
						
				saved = False
				dragging = False
					
			elif event.pos[1] <= 500:
				if selectedObject != -1:
					levelObjects.append(Object(objects[selectedObject].name,pygame.Rect(event.pos[0]-window.x_offset,event.pos[1]-window.y_offset,objects[selectedObject].pos[2],objects[selectedObject].pos[2])))
					saved = False
				else:
					for i in range(len(levelObjects)):
						if levelObjects[i].pos.collidepoint(event.pos):						
							for j in range(len(objects)):
								if objects[j].name == levelObjects[i].name:
									selectedObject = j
									break
							if i+1 < len(levelObjects):
								levelObjects = levelObjects[:i]+levelObjects[i+1:]	
							else:
								levelObjects = levelObjects[:i]
							
							dragging = False
							antagonistSelect = False
							selectedTile = -1
							antagonistAI = False
							protagonistMove = False
							break
							
				if antagonistSelect:
					if antagonistAI:
						levelAntagonists[-1].AI.append((event.pos[0]-event.pos[0]%tilewidth-window.x_offset+tilewidth/2,event.pos[1]-event.pos[1]%tilewidth-window.y_offset+tilewidth/2))
						if ctrlPressed:
							levelAntagonists[-1].AI.append((-1,-1))
						saved = False
					else:
						levelAntagonists.append(Antagonist(pygame.Rect(event.pos[0]-window.x_offset,event.pos[1]-window.y_offset,tilewidth,tilewidth),[]))
						saved = False
						antagonistAI = True
				else:
					for i in range(len(levelAntagonists)):
						if levelAntagonists[i].pos.collidepoint(event.pos):						
							antagonistSelect = True
							if i+1 < len(levelAntagonists):
								levelAntagonists = levelAntagonists[:i]+levelAntagonists[i+1:]	
							else:
								levelAntagonists = levelAntagonists[:i]
							
							dragging = False
							selectedObject = -1
							selectedTile = -1
							antagonistAI = False
							protagonistMove = False
							pieMove = False
							break
							
				if protagonistMove:
					protagonistPos = (event.pos[0]-window.x_offset,event.pos[1]-window.y_offset)
					selectedTile = -1
					selectedObject = -1
					antagonistSelect = False
					dragging = False
					antagonistAI = False
					protagonistMove = False
					pieMove = False
					saved = False
					
				if pieMove:
					piePos = (event.pos[0]-window.x_offset,event.pos[1]-window.y_offset)
					selectedTile = -1
					selectedObject = -1
					antagonistSelect = False
					dragging = False
					antagonistAI = False
					protagonistMove = False
					pieMove = False
					saved = False
						
		if event.type == QUIT:
			pygame.quit()
			if not saved:
				if raw_input("Save y/n?") in ["y","Y"]:
					save()
			sys.exit()	