import pygame, sys, random
from pygame.locals import *

levelPath = "res/levels/"
tilePath = "res/tiles/"
spritePath = "res/sprites/"
objectPath = "res/objects/"
screenshotPath = "res/screenshots/"
GUIpath = "res/gui/"

#keysets = [[K_x,K_c,K_z,K_a]] #default
#keysets = [[K_a,K_d,K_j,K_h]] WASD
keysets = [[K_LEFT,K_RIGHT,K_SPACE,K_LCTRL]] #arrowkeys
#You get the idea.  [[<move left>, <move right>, <jump>, <action>]]

numPlayers = 1

tilewidth = 32

g = 1.5

pygame.init()
pygame.mouse.set_visible(False)

numbers = []
levelNumIcon = pygame.image.load(GUIpath+"level.png")
levelNumIcon.set_colorkey((255,255,255))

slash = pygame.image.load(GUIpath+"slash.png")
slash.set_colorkey((255,255,255))

for i in range(10):
	numbers.append(pygame.image.load(GUIpath+str(i)+".png"))
	numbers[-1].set_colorkey((255,255,255))

class Window:
	def __init__(self,screenRes,GUIres):
		self.screenRes = screenRes
		self.window = pygame.display.set_mode(GUIres)
		self.screen = pygame.Surface(screenRes)
		self.gui = pygame.Surface(GUIres)
		self.gui.set_colorkey((255,255,255))
		self.x_offset = 0
		self.y_offset = 0
		self.screen.fill((0,0,0))
		self.gui.fill((255,255,255))
		
	def update(self):
		self.window.blit(self.screen, (0,0))
		self.window.blit(self.gui, (0,0))
		pygame.display.flip()
		self.screen.fill((0,0,0))
		self.gui.fill((255,255,255))
		
	def blitScreen(self,image,pos):
		self.screen.blit(image,(pos[0]+self.x_offset,pos[1]+self.y_offset))
		
	def blitGUI(self,image,pos):
		self.gui.blit(image,pos)	
		
	def centerOn(self,object):
		if object.pos[0]+self.x_offset < self.screenRes[0]/2:
			self.x_offset += (self.screenRes[0]/2-(object.pos[0]+self.x_offset))/4
		if object.pos[0]+self.x_offset > self.screenRes[0]/2:
			self.x_offset -= (object.pos[0]+self.x_offset-self.screenRes[0]/2)/4
		if object.pos[1]+self.y_offset < self.screenRes[1]/2:
			self.y_offset += (self.screenRes[01]/2-(object.pos[1]+self.y_offset))/4
		if object.pos[1]+self.y_offset > self.screenRes[1]/2:
			self.y_offset -= (object.pos[1]+self.y_offset-self.screenRes[1]/2)/4
			
	def getCenteredOn(self,object):
		return (((object.pos[0]+self.x_offset - self.screenRes[0]/2) <= 5) and ((object.pos[0]+self.x_offset - self.screenRes[0]/2) >= -5)) and (((object.pos[1]+self.y_offset - self.screenRes[1]/2) <= 5) and ((object.pos[1]+self.y_offset - self.screenRes[1]/2) >= -5))
			
	def getCenter(self):
		return self.window.get_width()/2
		
		
window = Window((500,500),(500,500))
		
class Tile:
	def __init__(self,name,transparent,pos):
		self.name = name
		self.pos = pos
		self.transparent = transparent
		self.image = pygame.image.load(tilePath+self.name+".png")
		self.image = pygame.transform.scale(self.image,(tilewidth,tilewidth))

class Dummy:
	def __init__(self):
		self.tiles = [Tile("0",True,pygame.Rect(0,0,0,0))]	
		self.pos = pygame.Rect(0,0,0,0)	

dummyCharacter = Dummy()		
						
class Object:
	def __init__(self,name,pos,level):
		self.name = name
		self.image = pygame.image.load(objectPath+self.name+".png")
		self.image = pygame.transform.scale(self.image,(self.image.get_width()*2,self.image.get_height()*2))
		self.image.set_colorkey((0,0,0))
		self.pos = pos	
		self.level = level
		self.velocity = 0.0
		self.character = dummyCharacter
		
	def update(self):
		if self.character == dummyCharacter:
			self.velocity += g
			
			for i in range(int(self.velocity)):
				self.pos[1] += 1
				if self.getCollide():
					self.pos[1] -= 1
					self.velocity = 0.0
					break
					
		else:
			self.allignToCharacter()
				
		window.blitScreen(self.image,self.pos)
		
	def allignToCharacter(self):
		self.pos[0] = self.character.pos[0]+self.character.pos[2]/2-self.pos[2]/2
		self.pos[1] = self.character.pos[1]-self.pos[3]
				
	def getCollide(self):
		for row in self.level.tiles:
			for tile in row:
				if self.pos.colliderect(tile.pos) and not tile.transparent:
					#pygame.draw.rect(window.screen,(0,0,0),tile.pos,1)
					return True
					
		for object in self.level.objects:
			if self.pos != object.pos:
				if self.pos.colliderect(object.pos):
					#pygame.draw.rect(window.screen,(0,0,0),tile.pos,1)
					return True	
							
		if not self.name == "pie":			
			return self.pos.colliderect(self.level.pie.pos)
			
		else:
			return False
		
class Enemy:
	def __init__(self,character,ai):
		self.character = character
		self.ai = ai
		self.aiTick = 0
		for i in range(len(self.ai)):
			self.ai[i] = int(self.ai[i])
			if self.ai[i] != -1:
				self.ai[i] *= 2
	def update(self):
		if not self.character.level.complete and self.character.injuryCount == -1:
			if self.aiTick < len(self.ai):
				if self.ai[self.aiTick] == -1:
					if self.character.jump():
						self.aiTick += 1
				else:
					distance = self.character.pos.centerx - self.ai[self.aiTick]
					
					if distance > 2:
						self.character.moving = -1
						
					elif distance < -2:
						self.character.moving = 1
						
					else:
						self.character.moving = 0
						self.aiTick += 1			
		
			else:
				if random.randint(0,200) == 1:
					self.character.moving = random.randint(-1,1)	
					
			if self.character.moving != 0:
				self.character.move(self.character.moving,0)
				if self.character.getCollide():
					self.character.move(-self.character.moving,0)
					self.character.jump()
				else:
					self.character.move(-self.character.moving,0)		
					
		self.character.update()
		
	def getVictory(self):
		self.character.move(self.character.moving,0)
		if self.character.pos.colliderect(self.character.level.pie.pos):
			self.character.move(-self.character.moving,0)
			self.character.carrying = True
			self.character.object = self.character.level.pie
			self.character.level.pie.character = self.character
			return True
		else:
			self.character.move(-self.character.moving,0)
			return False

dummyObject = Dummy()			
					
class Character:
	def __init__(self, name, pos, level):
		self.name = name
		self.level = level
		self.pos = pos
		self.pos[0] *= 2
		self.pos[1] *= 2
		self.pos[2] -= self.pos[2]/2
		self.pos[3] -= 1
		self.direction = 1
		self.tick = 0
		self.velocity = 0.0
		self.moving = 0
		self.images = []
		self.blood = []
		self.attack = False
		self.carrying = False
		self.object = dummyObject
		self.injuryCount = -1
		self.injuryDirection = 0
		modes = ["","_carry"]
		for j in range(len(modes)):  #add extra images here
			secondaryModes = ["stand","run1","run2"]
			for i in range(len(secondaryModes)):
				self.images.append(pygame.image.load(spritePath+self.name+"_"+secondaryModes[i]+modes[j]+".png"))
				
			for i in range(3):
				self.images.append(pygame.transform.flip(self.images[j*len(secondaryModes)*2+i],True,False))
				
		self.images.append(pygame.image.load(spritePath+self.name+"_dead.png"))
		self.images.append(pygame.transform.flip(self.images[-1],True,False))		
				
		self.images.append(pygame.image.load(spritePath+self.name+"_attack.png"))
		self.images.append(pygame.transform.flip(self.images[-1],True,False))
			
		for i in range(len(self.images)-2):
			self.images[i] = pygame.transform.scale(self.images[i],(tilewidth,tilewidth))
		
		for i in range(-2,0):	
			self.images[i] = pygame.transform.scale(self.images[i],(tilewidth*2,tilewidth))
			
		for i in range(len(self.images)):
			self.images[i].set_colorkey((255,255,255))
		
		self.blood.append(pygame.image.load(spritePath+"blood.png"))
		self.blood.append(pygame.transform.flip(self.blood[0],True,False))			
				
		for i in range(len(self.blood)):
			self.blood[i] = pygame.transform.scale(self.images[i],(tilewidth,tilewidth))
			self.blood[i].set_colorkey((255,255,255))
			
	def update(self):
		self.velocity += g
		fallDirection = 1
		if self.velocity < 0:
			fallDirection = -1
			
		for i in range(int(self.velocity*fallDirection)):
			self.move(0,fallDirection)
			if self.carrying:
				self.object.allignToCharacter()
			
			if self.getCollide():
				self.move(0,-fallDirection)
				if self.carrying:
					self.object.allignToCharacter()
				self.velocity = 0.0
				break
	
		if self.injuryCount == 0:
			self.pos[0] -= self.pos[2]/4
			if self.direction == -1:
				window.blitScreen(self.images[-3],self.pos)	
			else:
				window.blitScreen(self.images[-4],self.pos)
			self.pos[0] += self.pos[2]/4
			return
	
		self.tick += 1
		self.tick %= 9
		
		for i in range(4):		
			self.move(self.moving,0)
			if self.carrying:
				self.object.allignToCharacter()
			if self.getCollide():
				self.move(-self.moving,0)
				if self.carrying:
					self.object.allignToCharacter()
				break
				
		if not self.moving == 0:
			self.direction = self.moving
		
		if self.attack:
			if self.tick == 3:
				self.attack = False
				
			if self.direction == 1:
				self.move(-self.pos[2]/4,0)
				window.blitScreen(self.images[-2],self.pos)
				self.move(self.pos[2]/4,0)
				
			else:
				self.move(-(self.pos[2]/4+tilewidth),0)
				window.blitScreen(self.images[-1],self.pos)
				self.move(self.pos[2]/4+tilewidth,0)
				
			return
			
		imageNumOffset = 0

		if self.carrying:
			imageNumOffset += 6
			
		if self.direction == -1:
			imageNumOffset += 3
		
		self.pos[0] -= self.pos[2]/4
					
		if self.velocity >= 1 or self.velocity < 0 or self.moving == 0:
			window.blitScreen(self.images[0+imageNumOffset],self.pos)
		else:
			window.blitScreen(self.images[self.tick/3+imageNumOffset],self.pos)
		
		if self.injuryCount != -1:
			window.blitScreen(self.blood[self.injuryDirection],self.pos)
			self.injuryCount -= 1	
			
		self.pos[0] += self.pos[2]/4				
		#pygame.draw.rect(window.screen,(0,0,0),self.pos,1)
				
	def getCollide(self):
		for row in self.level.tiles:
			for tile in row:
				if self.pos.colliderect(tile.pos) and not tile.transparent:
					#pygame.draw.rect(window.screen,(0,0,0),tile.pos,1)
					return True
					
		for object in self.level.objects:
			if self.pos.colliderect(object.pos):
				#pygame.draw.rect(window.screen,(0,0,0),tile.pos,1)
				return True	
				
		if self.carrying:
			return self.object.getCollide()		
					
		return self.pos.colliderect(self.level.pie.pos)
		
	def getObject(self):					
		for object in self.level.objects:
			if self.pos.colliderect(object.pos):
				return object		
					
		return dummyObject
	
	def move(self,x,y):
		self.pos[0] += x
		self.pos[1] += y	
		
	def jump(self):
		jumped = False
		self.move(0,1)
		if self.getCollide():
			self.velocity = -15
			jumped = True
		self.move(0,-1)
		
		return jumped
			
	def action(self):
		if self.attack:
			return	
	
		if self.carrying:
			if self.direction == 1:
				self.object.pos[0] = self.pos[0] + self.pos[2] + 2				
			else:
				self.object.pos[0] = self.pos[0] - self.object.pos[2] - 2
				
			self.object.pos[1] = self.pos[1] + self.pos[3] - self.object.pos[3] - 2
			
			if self.object.getCollide():
				self.object.allignToCharacter()
			else:
				
				self.object.character = dummyCharacter
				self.object = dummyObject
				
				self.carrying = False
			
		else:
			self.move(self.direction,0)
			self.object = self.getObject()
			if not self.object == dummyObject:
				tempPos = pygame.Rect(self.object.pos[0],self.object.pos[1],self.object.pos[2],self.object.pos[3])
				self.object.character=self	
				self.object.allignToCharacter()
				if self.object.getCollide():
					self.object.pos = tempPos
					self.object.character = dummyCharacter
					self.object = dummyObject
				else:
					self.carrying = True		
			self.move(-self.direction,0)
			
			if not self.carrying:
				attackRect = pygame.Rect(0,self.pos[1],tilewidth,tilewidth)	
				
				if self.direction  < 0:
					attackRect[0] = self.pos[0]-tilewidth
				else:		
					attackRect[0] = self.pos[0]+tilewidth
					
				for enemy in self.level.enemies:
					if enemy.character.pos.colliderect(attackRect) and enemy.character.injuryCount == -1:
						enemy.character.injure(self.direction)
					
				self.attack = True
				self.tick = 0
				
	def injure(self,direction):
		if direction >= 0:
			self.injuryDirection = 0
		else:
			self.injuryDirection = 1
		
		self.moving = 0	
		self.injuryCount = 3		
		
#set up tiles

tiles = []

f = open(tilePath+"description.des", 'r')
lines = f.readlines()
f.close()

for line in lines:
	line = line.split(" ")
	tiles.append(Tile(line[0],(line[1] == "transparent"),0))	
	
objects = []

f = open(objectPath+"description.des", 'r')
lines = f.readlines()
f.close()

for line in lines:
	line = line.split(" ")
	objects.append(Object(line[0],pygame.Rect(0,0,int(line[1]),int(line[2])),Dummy()))
					
class Level:
	def __init__(self, number, game):
		self.number = number
		self.game = game
		self.tiles = []
		self.objects = []
		self.enemies = []
		self.players = []
		self.complete = False
		
		fileName = levelPath+str(self.number)+".lvl"
		f = open(fileName, 'r')
			
		lines = f.readlines()
		f.close()
		
		y = 0
		for i in range(int(lines[0])):
			line = lines[1+i].split(" ")
			row = []
			x = 0
			for tile in line:
				if tile != "\n":
					row.append(Tile(tiles[int(tile)].name,tiles[int(tile)].transparent,pygame.Rect(x*tilewidth,y*tilewidth,tilewidth,tilewidth)))
					x += 1
			self.tiles.append(row)
			y += 1
			
		start = int(lines[0])+1		
		for i in range(start,start+int(lines[start])):
			line = lines[1+i].split(" ")
			self.objects.append(Object(line[0],pygame.Rect(int(line[1])*2,int(line[2])*2,0,0),self))
			for object in objects:
				if object.name == self.objects[-1].name:
					self.objects[-1].pos[2] = object.pos[2]*2
					self.objects[-1].pos[3] = object.pos[3]*2
					break
					
		start += int(lines[start])+1
			
		for i in range(start,start+int(lines[start])):
			line = lines[1+i].split(" ")
			self.enemies.append(Enemy(Character("clown",pygame.Rect(int(line[0]),int(line[1]),tilewidth,tilewidth),self),line[2:]))
			
		index = start+int(lines[start])+1
		line = lines[index].split(" ")
		
		self.players.append(Character("dude",pygame.Rect(int(line[0]),int(line[1]),tilewidth,tilewidth),self))
		
		index += 1
		line = lines[index].split(" ")
		
		self.pie = Object("pie",pygame.Rect(int(line[0])*2,int(line[1])*2,32,16),self)
		
	def update(self):
		"""
		for y in range(len(self.tiles)):
			for x in range(len(self.tiles[y])):
				window.blitScreen(self.tiles[y][x].image,(x*tilewidth,y*tilewidth))
		"""
		for row in self.tiles:
			for tile in row:
				window.blitScreen(tile.image,tile.pos)	
				
		for object in self.objects:
			object.update()	
		
		self.pie.update()	
				
		for enemy in self.enemies:
			if not self.complete:
				if enemy.getVictory():
					self.endLevel(False,self.pie)	
			enemy.update()	
			
		for player in self.players:
			player.update();
		
			window.centerOn(player)	
		
		levelNum = pygame.Surface((70,12))
		levelNum.fill((255,255,255))
		
		levelNum.blit(levelNumIcon,(0,0))
		
		levelNumStr = str(self.number)
		for s in range(len(levelNumStr)):
			levelNum.blit(numbers[int(levelNumStr[s])],(levelNumIcon.get_width()+s*12,0))
		
		window.blitGUI(levelNum,(10,10))	
		
		
		clownNum = pygame.Surface((70,12))
		clownNum.fill((255,255,255))
		
		offSet = 0
				
		clownNumStr = str(len(self.enemies))
		
		deadNum = 0
		for enemy in self.enemies:
			if enemy.character.injuryCount == 0:
				deadNum += 1
				
		deadNum = str(deadNum)
		
		for s in range(len(deadNum)):
			clownNum.blit(numbers[int(deadNum[s])],(offSet,0))
			offSet += numbers[int(deadNum[s])].get_width()
		
		clownNum.blit(slash,(offSet,0))
		
		offSet += slash.get_width()
		
		for s in range(len(clownNumStr)):
			clownNum.blit(numbers[int(clownNumStr[s])],(offSet,0))
			offSet += numbers[int(clownNumStr[s])].get_width()
		
		window.blitGUI(clownNum,(window.window.get_width()-10-offSet,10))	
			
		if not self.complete:	
			if self.testComplete():
				self.endLevel(True,self.players[0])
					
	def testComplete(self):
		for enemy in self.enemies:
			if enemy.character.injuryCount != 0:
				return False
				
		return True
		
	def endLevel(self,won,victor):
		self.complete = True	
		"""
		if won:
			message = "LEVEL COMPLETE!"
		
		font = pygame.font.SysFont(None,50) 	
				
		messageImage = font.render(message,True,(0,0,0),(255,255,255))
		messageImage.set_colorkey((255,255,255))
		messageRect = pygame.Rect(window.getCenter()-messageImage.get_width()/2,50,messageImage.get_width(),messageImage.get_height())	
		"""
		
		if won:
			messageImage = pygame.image.load(GUIpath+"complete.png")
			
		else:
			messageImage = pygame.image.load(GUIpath+"failed.png")
		
		messageImage = pygame.transform.scale(messageImage,(messageImage.get_width()*2,messageImage.get_height()*2))
			
		messageImage.set_colorkey((255,255,255))
		messageRect = pygame.Rect(window.getCenter()-messageImage.get_width()/2,50,messageImage.get_width(),messageImage.get_height())	
				
		for enemy in self.enemies:
			enemy.character.moving = 0	
			
		for player in self.players:
			player.moving = 0
		
		tick = 40
		noAction = True
			
		while tick > 0 and noAction:
			tick -= 1
			self.update()
			window.blitGUI(messageImage,messageRect)
			window.centerOn(victor)	
			window.update()
		
			for event in pygame.event.get():
				if event.type == KEYUP:
					noAction = False		
							
				if event.type == QUIT:
					pygame.quit()
					sys.exit()
			
			self.game.clock.tick(20)			
							
		if won:			
			self.game.nextLevel()
		else:
			self.game.restartLevel()	
				
class Game():
	def __init__(self,numLevels):	
		self.clock = pygame.time.Clock()
		self.levelNumber = 1
		self.numLevels = numLevels
		self.level = Level(self.levelNumber,self)
		
	def nextLevel(self):
		self.levelNumber += 1
		if self.levelNumber > self.numLevels:
			
			pygame.quit()
			sys.exit()
			
		self.level = Level(self.levelNumber,self)
		
	def restartLevel(self):
		self.level = Level(self.levelNumber,self)
		
	def loop(self):
		while True:
		
			#draw screen
			
			self.level.update()
								
			window.update()
			
			#handle events
			
			for event in pygame.event.get():
				if event.type == KEYUP:
					if event.key == K_ESCAPE:
						self.restartLevel()
						
					for i in range(numPlayers):
						if event.key == keysets[i][0] and self.level.players[i].moving == -1:
							self.level.players[i].moving = 0	
							
						if event.key == keysets[i][1] and self.level.players[i].moving == 1:
							self.level.players[i].moving = 0
							
					if event.key == K_F11:
						pygame.image.save(pygame.display.get_surface(),screenshotPath+raw_input("Save as?"))	
					
				if event.type == KEYDOWN:		
					for i in range(numPlayers):
						if event.key == keysets[i][0]:
							self.level.players[i].moving = -1	
							
						if event.key == keysets[i][1]:
							self.level.players[i].moving = 1
							
						if event.key == keysets[i][2]:
							self.level.players[i].jump()
								
						if event.key == keysets[i][3]:
							self.level.players[i].action()
					
				if event.type == QUIT:
					pygame.quit()
					sys.exit()	
			
			#tick
			self.clock.tick(20)		
game = Game(7)
game.loop()